/*
 * QEstEidUtil
 *
 * Copyright (C) 2011 Jargo Kõster <jargo@innovaatik.ee>
 * Copyright (C) 2011 Raul Metsma <raul@innovaatik.ee>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#pragma once

#include <QThread>

#include <QMetaType>
#include <QSharedDataPointer>

class SslCertificate;
class QSmartCardDataPrivate;

class QSmartCardData
{
public:
	enum PersonalDataType
	{
		SurName = 0,
		FirstName,
		MiddleName,
		FullName,
		Sex,
		Citizen,
		BirthDate,
		Id,
		DocumentId,
		Expiry,
		BirthPlace,
		IssueDate,
		ResidencePermit,
		Comment1,
		Comment2,
		Comment3,
		Comment4,
		Email
	};
	enum PinType
	{
		Pin1Type,
		Pin2Type,
		PukType
	};
	enum CardType
	{
		Unknown,
		IDCard,
		DigiID,
		Tempel
	};
	enum CardVersion
	{
		UnknownVersion = -1,
		Ver_1_0 = 0,
		Ver_1_0_2007,
		Ver_1_1,
		Ver_3_0
	};

	QSmartCardData();
	QSmartCardData( const QSmartCardData &other );
	~QSmartCardData();
	QSmartCardData operator=( const QSmartCardData &other );

	void clear();

	QString card() const;
	void setCard( const QString &card );

	QStringList cards() const;
	void setCards( const QStringList &cards );

	CardType cardType() const;

	QStringList readers() const;
	void setReaders( const QStringList &readers );

	QVariant data( PersonalDataType type ) const;
	void setData( PersonalDataType type, const QVariant &data );

	bool isPinpad() const;
	void setPinpad( bool pinpad );

	bool isValid() const;

	SslCertificate authCert() const;
	void setAuthCert( const SslCertificate &cert );

	SslCertificate signCert() const;
	void setSignCert( const SslCertificate &cert );

	quint8 retryCount( PinType type ) const;
	void setRetryCount( PinType type, quint8 count );

	quint16 usageCount( PinType type ) const;
	void setUsageCount( PinType type, quint16 count );

	CardVersion version() const;
	void setVersion( CardVersion version );

	static QString typeString( PinType type );

private:
	QSharedDataPointer<QSmartCardDataPrivate> d;
};

Q_DECLARE_METATYPE(QSmartCardData)



class QSmartCardPrivate;
class QSmartCard: public QThread
{
    Q_OBJECT
public:
	enum ErrorType
	{
		NoError,
		UnknownError,
		BlockedError,
		CancelError,
		DifferentError,
		LenghtError,
		ValidateError
	};

	explicit QSmartCard( QObject *parent = 0 );
	~QSmartCard();

	ErrorType change( QSmartCardData::PinType type, const QString &newpin, const QString &pin );
	QSmartCardData data() const;
	QHash<QString,QString> diagnostics() const;
	Qt::HANDLE key();
	QSmartCard::ErrorType login( QSmartCardData::PinType type );
	void logout();
	void reload();
	ErrorType unblock( QSmartCardData::PinType type, const QString &pin, const QString &puk );

signals:
	void dataChanged();
	void eventStarted();

private Q_SLOTS:
	void selectCard( const QString &card );

private:
	void run();

	QSmartCardPrivate *d;

	friend class CertUpdate;
};
